{% extends "registration/base.html" %}

{% block content %}
<div class="login-box password-reset">
    {% block login_logo %}
    <div class="login-logo">
        <a>{{ project_gui_name }}</a>
    </div>
    {% endblock %}

    <div class="login-box-body rounded" style="text-align: center">
        <p style="color: #000000; margin-bottom: 25px">Your password has been set.  You may go ahead and log in now.</p>
        <button class="btn btn-primary"><a style="color: #ffffff" href="{{ login_url }}">Log in</a></button>
    </div>
</div>

{% endblock %}