{% extends "registration/base.html" %}


{% block style %}
    {{ block.super }}
    <style>
        .password-reset-label {
            color: #000000;
            margin-right: 10px;
            padding-right: 10px;
            float: left
        }
    </style>
{% endblock %}

{% block content %}
<div class="login-box password-reset">
    {% block login_logo %}
    <div class="login-logo">
        <a>{{ project_gui_name }}</a>
    </div>
    {% endblock %}

    <div class="login-box-body rounded">
        {% if validlink %}

        <p>Please enter your new password twice so we can verify you typed it in correctly.</p>
        <form method="post">
            {% csrf_token %}
            <fieldset class="module aligned">
                <div style="margin-bottom: 25px">
                    <div style="margin: 0 auto 25px auto;width: min-content;clear:both; text-align:right; line-height:25px;">
                        {{ form.new_password1.errors }}
                        <label for="id_new_password1" class="password-reset-label">New password:</label>
                        {{ form.new_password1 }}
                    </div>
                    <div style="margin: 0 auto 25px auto;width: min-content;clear:both; text-align:right; line-height:25px;">
                        {{ form.new_password2.errors }}
                        <label for="id_new_password2" class="password-reset-label">Confirm password:</label>
                        {{ form.new_password2 }}
                    </div>
                </div>
                <div style="text-align: center">
                    <input type="submit" value="Change my password" class="btn btn-primary">
                </div>

            </fieldset>
        </form>
        {% else %}

        <p>The password reset link was invalid, possibly because it has already been used.  Please request a new password reset.</p>
        {% endif %}
    </div>
</div>


{% endblock %}