{% extends "registration/base.html" %}

{% block content %}
<div class="login-box password-reset">
    {% block login_logo %}
    <div class="login-logo">
        <a>{{ project_gui_name }}</a>
    </div>
    {% endblock %}

    <div class="login-box-body rounded">
        <p style="color: #000000">We've emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.</p>
        <p style="color: #000000">If you don't receive an email, please make sure you've entered the address you registered with, and check your spam folder.</p>
    </div>
</div>
{% endblock %}