{% extends "base.html" %}
{% load request_static %}

{% block style %}
    {{ block.super }}
    <style>
        @media (min-width: 600px) {
            .password-reset {
                width: 500px
            }
        }
    </style>
{% endblock %}

{% block common_sciprt %}
<script src="{% static_path 'bundle/auth~doc~spa.chunk.js' %}"></script>
<script src="{% static_path 'bundle/auth~spa.chunk.js' %}"></script>
<script src="{% static_path 'bundle/auth.js' %}"></script>
{% endblock %}

{% block body %}
<body class="{% block bodyclass %}hold-transition login-page{% endblock %}">

    {% block content %}
    {% endblock %}

</body>
{% endblock %}