{% extends "registration/base.html" %}

{% block content %}

<div class="login-box password-reset">
    {% block login_logo %}
    <div class="login-logo">
        <a>{{ project_gui_name }}</a>
    </div>
    {% endblock %}

    <div class="login-box-body rounded">
        <p style="color: #000000">Forgotten your password? Enter your email address below, and we'll email instructions for setting a new one.</p>

        <form method="post">
            {% csrf_token %}
            <fieldset class="module aligned">
                <div style="margin: 0 auto 25px auto;width: min-content;">
                    {{ form.email.errors }}
                    <label for="id_email" style="color: #000000; margin-right: 10px">Email address:</label>
                    {{ form.email }}
                </div>
                <div style="text-align: center">
                    <input type="submit" value='Reset my password' class="btn btn-primary">
                </div>
            </fieldset>
        </form>
    </div>
</div>

{% endblock %}